﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daStaticAnalyser.Analysers
{
    using daBase.Analyser;
    using daSyntaxAnalyser.SyntaxTree;
    using daStaticAnalyser.Analysers;
    using daBase.Objects;
    using daStaticAnalyser;
    [AnalyserDefaultSettings("Цикломатическая сложность.", "test", 0, "test")]
    public class CyclomaticAnalyser : BaseStaticAnalyser
    {
        /// <summary>
        /// Расчет цикломатической сложности оператора
        /// </summary>
        /// <param name="Element">узел, соответствующий </param>
        /// <returns>цикломатическая </returns>
        private int CalculateCyclomatic(ISyntaxTreeElement Element)
        {
            if (Element == null)
                return 0;
            int result = 0;
            if ((Element is CaseBlockStatementElement) ||
                (Element is TryStatementElement) ||
                (Element is RepeatStatementElement) ||
                (Element is ExceptOnStatementElement) ||
                (Element is ForStatementElement) ||
                (Element is TestStatementElement) ||
                (Element is WhileStatementElement))
                result = 1;
            foreach (ISyntaxTreeElement element in Element)
                result += CalculateCyclomatic(element);
            return result;
        }

        /// <summary>
        /// Расчет цикломатической сложности метода
        /// </summary>
        /// <param name="Body">узел, соответствующий телу метода</param>
        /// <returns>цикломатическая сложность</returns>
        private int CalculateCyclomatic(MethodBodyElement Body)
        {
            if (Body != null)
                return CalculateCyclomatic(Body.Statement) + 1;
            return 0;
        }
        // расчитанная цикломатическая сложность
        private int diff;

        protected override void DoAnalyse(SyntaxTreeArgs es)
        {
            if (es.Element is OuterMethodElement)
            {
                var OuterMethod = es.Element as OuterMethodElement;
                diff = CalculateCyclomatic(OuterMethod.Body);
                AddMetric("Метрика МакКейба(Упрощённое вычисление)", diff, 
                    (OuterMethod.Header as OuterProcedureHeadElement).Id.Text, 
                    (OuterMethod.Header as OuterProcedureHeadElement).Id.File,
                    (OuterMethod.Header as OuterProcedureHeadElement).Id.Line);

                
            }
        }
    }
}
