﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daBase.Objects
{
    /// <summary>
    /// Карта файла
    /// </summary>
    public class FileMap
    {
        // Карта
        private Dictionary<int, int> filemap;

        public List<int> EmptyStrings;//Список пустых строк

        // Последняя строка
        private int last;

        public int Last { get { return last; } }

        /// <summary>
        /// Добавить строку
        /// </summary>
        /// <param name="index">начало строки</param>
        /// <returns>номер добавленной строки</returns>
        public int AddLine(int index)
        {
            if ((last == 0) || (index > filemap[last]))
                filemap.Add(++last, index);
            return last;
        }

        /// <summary>
        /// Сбросить состояние карты
        /// </summary>
        public void Reset()
        {
            if (filemap == null)
            {
                filemap = new Dictionary<int, int>();
                EmptyStrings = new List<int>();
            }
            else
            {
                filemap.Clear();
                EmptyStrings.Clear();
            }
            last = 0;
            AddLine(0);
        }

        /// <summary>
        /// Поиск строки, в которую входит символ
        /// </summary>
        /// <param name="start">левая граница поиска</param>
        /// <param name="end">правая граница поиска</param>
        /// <param name="index">предмет поиска</param>
        /// <returns>номер строки</returns>
        private int FindKey(int start, int end, int index)
        {
            if (start == end)
                return start;
            else
            {
                int count = end - start + 1, center = start + count / 2;
                if (filemap[center] <= index)
                    return FindKey(center, end, index);
                else if (count == 2) return start;
                else return FindKey(start, center, index);
            }
        }

        /// <summary>
        /// Номер строки
        /// </summary>
        /// <param name="index">номер символа</param>
        /// <returns>номер строки, содержащей символ</returns>
        public int Line(int index)
        {
            if (filemap[last] <= index) return last;
            return FindKey(1, last, index);
        }

        /// <summary>
        /// Номер символа начала строки
        /// </summary>
        /// <param name="Line">номер строки</param>
        /// <returns>номер символа</returns>
        public int Index(int Line)
        {
            if (filemap.ContainsKey(Line))
                return filemap[Line];
            return filemap[last];
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        public FileMap()
        {
            Reset();
        }
    }
}
