﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daBase.Objects
{
    using Safe;
    using AnalysisErrors;

    /// <summary>
    /// Аргумент - исходный код
    /// </summary>
    public class SourceArgs : EventArgs
    {
        private AnalysedFileInfo file;
        private string source;

        /// <summary>
        /// Информация о проанализированном файле
        /// </summary>
        public AnalysedFileInfo File { get { return file; } }

        /// <summary>
        /// Исходный код
        /// </summary>
        public string Source { get { return source; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="File">информация о проанализированном файле</param>
        /// <param name="Source">исходный код файла</param>
        public SourceArgs(AnalysedFileInfo File, string Source)
        {
            file = File;
            source = Source;
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="File">информация о проанализированном файле</param>
        /// <param name="Map">карта файла</param>
        /// <param name="MetaTags">метатеги</param>
        /// <param name="Source">исходный код файла</param>
        public SourceArgs(string File, FileMap Map, SafeDictionary<string, string> MetaTags, string Source)
        {
            file = new AnalysedFileInfo(File, Map, MetaTags);
            source = Source;
        }
    }
}
