﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSemanticAnalyser.SemanticTree
{
    using daSyntaxAnalyser.SyntaxTree;

    /// <summary>
    /// Свойство dispatch-интерфейса
    /// </summary>
    public class DispinterfacePropertySemanticElement : UnitElementSemanticElement
    {
        private List<VarSemanticElement> indexes;
        private CommonTypeSemanticElement type;
        private bool isreadonly;
        private PropertyOptionSemanticElement dispid;
        private bool _default;

        /// <summary>
        /// Индексаторы свойства
        /// </summary>
        public List<VarSemanticElement> Indexes { get { return indexes; } set { indexes = value; } }

        /// <summary>
        /// Тип свойства
        /// </summary>
        public CommonTypeSemanticElement Type { get { return type; } set { type = value; } }

        /// <summary>
        /// Является ли свойство свойством только для чтения
        /// </summary>
        public bool IsReadOnly { get { return isreadonly; } set { isreadonly = value; } }

        /// <summary>
        /// Опция dispid
        /// </summary>
        public PropertyOptionSemanticElement Dispid { get { return dispid; } set { dispid = value; } }

        /// <summary>
        /// Опция default/nodefault
        /// </summary>
        public bool Default { get { return _default; } set { _default = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="Name">имя элемента</param>
        /// <param name="Parent">родитель свойства</param>
        /// <param name="Unit">модуль, содержащий свойство</param>
        /// <param name="Indexes">индексаторы свойства</param>
        /// <param name="Type">тип свойства</param>
        /// <param name="IsReadOnly">является ли свойство свойством только для чтения</param>
        /// <param name="Dispid">опция dispid</param>
        /// <param name="Default">опция default</param>
        /// <param name="SyntaxElement">соответствующий элемент синтаксического дерева</param>
        public DispinterfacePropertySemanticElement(string Name, IChildSemanticElement Parent, FileSemanticElement Unit, List<VarSemanticElement> Indexes, CommonTypeSemanticElement Type, bool IsReadOnly, PropertyOptionSemanticElement Dispid, bool Default, ISyntaxTreeElement SyntaxElement)
            : base(Name, Parent, Unit, SyntaxElement)
        {
            indexes = (Indexes == null) ? new List<VarSemanticElement>() : Indexes;
            type = Type;
            isreadonly = IsReadOnly;
            dispid = Dispid;
            _default = Default;
        }
    }
}
