﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSemanticAnalyser.SemanticTree
{
    using daSyntaxAnalyser.SyntaxTree;

    /// <summary>
    /// Dispatch-интерфейс
    /// </summary>
    public class DispinterfaceTypeSemanticElement : TypeSemanticElement
    {
        private ConstExpressionElement guid;
        private List<DispinterfacePropertySemanticElement> properties;
        private List<MethodSemanticElement> methods;

        /// <summary>
        /// GUID dispatch-интерфейса
        /// </summary>
        public ConstExpressionElement GUID { get { return guid; } set { guid = value; } }

        /// <summary>
        /// Свойства dispatch-интерфейса
        /// </summary>
        public List<DispinterfacePropertySemanticElement> Properties { get { return properties; } set { properties = value; } }

        /// <summary>
        /// Методы dispatch-интерфейса
        /// </summary>
        public List<MethodSemanticElement> Methods { get { return methods; } set { methods = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="Name">имя элемента</param>
        /// <param name="Parent">родитель элемента семантического дерева</param>
        /// <param name="Unit">модуль, содержащий элемент</param>
        /// <param name="Packed">является ли тип запакованным</param>
        /// <param name="GUID">GUID dispatch-интерфейса</param>
        /// <param name="Properties">свойства dispatch-интерфейса</param>
        /// <param name="Methods">методы dispatch-интерфейса</param>
        /// <param name="SyntaxElement">соответствующий элемент синтаксического дерева</param>
        public DispinterfaceTypeSemanticElement(string Name, IChildSemanticElement Parent, FileSemanticElement Unit, bool Packed, ConstExpressionElement GUID, List<DispinterfacePropertySemanticElement> Properties, List<MethodSemanticElement> Methods, ISyntaxTreeElement SyntaxElement)
            : base(Name, Parent, Unit, Packed, SyntaxElement)
        {
            guid = GUID;
            properties = (Properties == null) ? new List<DispinterfacePropertySemanticElement>() : Properties;
            methods = (Methods == null) ? new List<MethodSemanticElement>() : Methods;
        }
    }
}
