﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSemanticAnalyser.SemanticTree
{
    using daSyntaxAnalyser.SyntaxTree;

    /// <summary>
    /// Интерфейс
    /// </summary>
    public class InterfaceTypeSemanticElement : TypeSemanticElement
    {
        private InterfaceTypeSemanticElement parentInterface;
        private ConstExpressionElement guid;
        private List<InterfacePropertySemanticElement> properties;
        private List<MethodSemanticElement> methods;

        /// <summary>
        /// Родительские интерфейсы
        /// </summary>
        public InterfaceTypeSemanticElement ParentInterface { get { return parentInterface; } set { parentInterface = value; } }

        /// <summary>
        /// GUID интерфейса
        /// </summary>
        public ConstExpressionElement GUID { get { return guid; } set { guid = value; } }

        /// <summary>
        /// Свойства интерфейса
        /// </summary>
        public List<InterfacePropertySemanticElement> Properties { get { return properties; } set { properties = value; } }

        /// <summary>
        /// Методы интерфейса
        /// </summary>
        public List<MethodSemanticElement> Methods { get { return methods; } set { methods = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="Name">имя элемента</param>
        /// <param name="Parent">родитель элемента семантического дерева</param>
        /// <param name="Unit">модуль, содержащий элемент</param>
        /// <param name="Packed">является ли тип запакованным</param>
        /// <param name="Parents">родительские интерфейсы</param>
        /// <param name="GUID">GUID интерфейса</param>
        /// <param name="Properties">свойства интерфейса</param>
        /// <param name="Methods">методы интерфейса</param>
        /// <param name="SyntaxElement">соответствующий элемент синтаксического дерева</param>
        public InterfaceTypeSemanticElement(string Name, IChildSemanticElement Parent, FileSemanticElement Unit, bool Packed, InterfaceTypeSemanticElement ParentInterface, ConstExpressionElement GUID, List<InterfacePropertySemanticElement> Properties, List<MethodSemanticElement> Methods, ISyntaxTreeElement SyntaxElement)
            : base(Name, Parent, Unit, Packed, SyntaxElement)
        {
            parentInterface = ParentInterface;
            guid = GUID;
            properties = (Properties == null) ? new List<InterfacePropertySemanticElement>() : Properties;
            methods = (Methods == null) ? new List<MethodSemanticElement>() : Methods;
        }
    }
}
