﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSemanticAnalyser.SemanticTree
{
    using daSyntaxAnalyser.SyntaxTree;

    /// <summary>
    /// Метод
    /// </summary>
    public class MethodSemanticElement : UnitElementSemanticElement
    {
        private MethodTypes type;
        private List<ParamSemanticElement> parameters;
        private List<MethodDirectiveSemanticElement> directives;
        private MethodRealizationSemanticElement realization;

        /// <summary>
        /// Тип метода (процедура, функция, конструктор ...)
        /// </summary>
        public MethodTypes Type { get { return type; } set { type = value; } }

        /// <summary>
        /// Список параметров
        /// </summary>
        public List<ParamSemanticElement> Params { get { return parameters; } set { parameters = value; } }

        /// <summary>
        /// Директивы метода
        /// </summary>
        public List<MethodDirectiveSemanticElement> Directives { get { return directives; } set { directives = value; } }

        /// <summary>
        /// Реализация метода
        /// </summary>
        public MethodRealizationSemanticElement Realization { get { return realization; } set { realization = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="Name">имя метода</param>
        /// <param name="Parent">родитель элемента семантического дерева</param>
        /// <param name="Unit">модуль, содержащий метод</param>
        /// <param name="Type">тип метода (процедура, функция, конструктор ...)</param>
        /// <param name="Params">список параметров метода</param>
        /// <param name="Directives">директивы метода</param>
        /// <param name="Realization">реализация метода</param>
        /// <param name="SyntaxElement">соответствующий элемент синтаксического дерева</param>
        public MethodSemanticElement(string Name, IChildSemanticElement Parent, FileSemanticElement Unit, MethodTypes Type, List<ParamSemanticElement> Params, List<MethodDirectiveSemanticElement> Directives, MethodRealizationSemanticElement Realization, ISyntaxTreeElement SyntaxElement)
            : base(Name, Parent, Unit, SyntaxElement)
        {
            type = Type;
            parameters = (Params == null) ? new List<ParamSemanticElement>() : Params;
            directives = (Directives == null) ? new List<MethodDirectiveSemanticElement>() : Directives;
            realization = Realization;
        }
    }
}
