﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSemanticAnalyser.SemanticTree
{
    using daSyntaxAnalyser.SyntaxTree;

    /// <summary>
    /// Тип-метод
    /// </summary>
    public class MethodTypeSemanticElement : TypeSemanticElement
    {
        private MethodTypes type;
        private List<ParamSemanticElement> parameters;
        private bool ofobject;
        private List<MethodDirectiveSemanticElement> directives;

        /// <summary>
        /// Тип метода (процедура, функция, конструктор ...)
        /// </summary>
        public MethodTypes Type { get { return type; } set { type = value; } }

        /// <summary>
        /// Список параметров
        /// </summary>
        public List<ParamSemanticElement> Params { get { return parameters; } set { parameters = value; } }

        /// <summary>
        /// Флаг наличия директивы of object
        /// </summary>
        public bool OfObject { get { return ofobject; } set { ofobject = value; } }

        /// <summary>
        /// Директивы метода
        /// </summary>
        public List<MethodDirectiveSemanticElement> Directives { get { return directives; } set { directives = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="Name">имя метода</param>
        /// <param name="Parent">родитель элемента семантического дерева</param>
        /// <param name="Unit">модуль, содержащий метод</param>
        /// <param name="Type">тип метода (процедура, функция, конструктор ...)</param>
        /// <param name="Params">список параметров метода</param>
        /// <param name="OfObject">флаг наличия директивы of object</param>
        /// <param name="Directives">директивы метода</param>
        /// <param name="SyntaxElement">соответствующий элемент синтаксического дерева</param>
        public MethodTypeSemanticElement(string Name, IChildSemanticElement Parent, FileSemanticElement Unit, bool Packed, MethodTypes Type, List<ParamSemanticElement> Params, bool OfObject, List<MethodDirectiveSemanticElement> Directives, ISyntaxTreeElement SyntaxElement)
            : base(Name, Parent, Unit, Packed, SyntaxElement)
        {
            type = Type;
            parameters = (Params == null) ? new List<ParamSemanticElement>() : Params;
            ofobject = OfObject;
            directives = (Directives == null) ? new List<MethodDirectiveSemanticElement>() : Directives;
        }
    }
}
