﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSemanticAnalyser.SemanticTree
{
    using daSyntaxAnalyser.SyntaxTree;

    /// <summary>
    /// Элемент блока case внутри записи
    /// </summary>
    public class RecordCaseElementSemanticElement : UnitElementSemanticElement
    {
        private List<ExpressionElement> _params;
        private List<VarSemanticElement> vars;

        /// <summary>
        /// Параметры элемента
        /// </summary>
        public List<ExpressionElement> Params { get { return _params; } set { _params = value; } }

        /// <summary>
        /// Переменные, объявленные в элементе
        /// </summary>
        public List<VarSemanticElement> Vars { get { return vars; } set { vars = vars; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="Name">имя элемента</param>
        /// <param name="Parent">родитель элемента семантического дерева</param>
        /// <param name="Unit">модуль, содержащий элемент</param>
        /// <param name="Params">параметры элемента</param>
        /// <param name="Vars">переменные, объявленные в элементе</param>
        /// <param name="SyntaxElement">соответствующий элемент синтаксического дерева</param>
        public RecordCaseElementSemanticElement(string Name, IChildSemanticElement Parent, FileSemanticElement Unit, List<ExpressionElement> Params, List<VarSemanticElement> Vars, ISyntaxTreeElement SyntaxElement)
            : base(Name, Parent, Unit, SyntaxElement)
        {
            _params = (Params == null) ? new List<ExpressionElement>() : Params;
            vars = (Vars == null) ? new List<VarSemanticElement>() : Vars;
        }
    }
}
