﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSemanticAnalyser.SemanticTree
{
    using daSyntaxAnalyser.SyntaxTree;

    /// <summary>
    /// Тип запись
    /// </summary>
    public class RecordTypeSemanticElement : TypeSemanticElement
    {
        private List<VarSemanticElement> vars;
        private List<ClassPropertySemanticElement> properties;
        private List<MethodSemanticElement> methods;
        private RecordCaseSemanticElement _case;

        /// <summary>
        /// Поля записи
        /// </summary>
        public List<VarSemanticElement> Vars { get { return vars; } set { vars = value; } }

        /// <summary>
        /// Свойства записи
        /// </summary>
        public List<ClassPropertySemanticElement> Properties { get { return properties; } set { properties = value; } }

        /// <summary>
        /// Методы записи
        /// </summary>
        public List<MethodSemanticElement> Methods { get { return methods; } set { methods = value; } }

        /// <summary>
        /// Блок case (опциональноы)
        /// </summary>
        public RecordCaseSemanticElement Case { get { return _case; } set { _case = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="Name">имя записи</param>
        /// <param name="Parent">родитель записи</param>
        /// <param name="Unit">модуль, содержащий объявление типа записи</param>
        /// <param name="Packed">является ли запись запакованной</param>
        /// <param name="Vars">переменные записи</param>
        /// <param name="Properties">свойства записи</param>
        /// <param name="Methods">методы записи</param>
        /// <param name="Case">блок case (опциональноы)</param>
        /// <param name="SyntaxElement">соответствующий элемент синтаксического дерева</param>
        public RecordTypeSemanticElement(string Name, IChildSemanticElement Parent, FileSemanticElement Unit, bool Packed, List<VarSemanticElement> Vars, List<ClassPropertySemanticElement> Properties, List<MethodSemanticElement> Methods, RecordCaseSemanticElement Case, ISyntaxTreeElement SyntaxElement)
            : base(Name, Parent, Unit, Packed, SyntaxElement)
        {
            vars = (Vars == null) ? new List<VarSemanticElement>() : Vars;
            properties = (Properties == null) ? new List<ClassPropertySemanticElement>() : Properties;
            methods = (Methods == null) ? new List<MethodSemanticElement>() : Methods;
            _case = Case;
        }
    }
}
