﻿using System;
using System.Collections.Generic;
using System.Text;
using daStaticAnalyser.Analysers;
using daBase.Analyser;
using daBase.Objects;
using daSyntaxAnalyser.SyntaxTree;
using daStaticAnalyser;

namespace daStaticAnalyser.Analysers
{
    [AnalyserDefaultSettings("Анализатор глобальных переменных", "Обнаружена глобальная переменная", 0, "Описание ошибки")]
    public class GlobalVarAnalyzer : BaseStaticAnalyser
    {

        string CurFile = "";
        int КоличествоГлобальныхПеременных;

        protected override void DoAnalyse(SyntaxTreeArgs e)
        {
            if (CurFile == "") CurFile = e.Element.StartInfo.File;
            if (e.Element.StartInfo.File != CurFile)
            {
                //AddMetric("Количество глобальных переменных", КоличествоГлобальныхПеременных, CurFile, CurFile, 0);
                //КоличествоГлобальныхПеременных = 0;
                CurFile = e.Element.StartInfo.File;
            }
            if (e.Element is UnitElement)
            {
                UnitElement ue = e.Element as UnitElement;

                foreach (ISyntaxTreeElement intf in ue.Interfaces)
                {
                    var vars = intf as VarDeclarationsElement;
                    var types = intf as TypeDeclarationsElement;
                    if (vars != null)
                    {
                        foreach (var v in vars)
                        {
                            AddError(v.StartInfo, "Глобальная переменная " + v.ToString());
                            КоличествоГлобальныхПеременных++;
                        }
                    }
                    else if(types != null)
                    {
                        foreach (var type in types.Declarations)
                        {
                            var def = type.Definition as FullClassDefinitionElement;
                            if (def == null) continue;
                            foreach (var sec in def.Sections)
                            {
                                var temp = sec as ClassSectionElement;
                                if (temp == null) continue;
                                foreach (var elt in temp.Elements)
                                {
                                    vars = elt as VarDeclarationsElement;
                                    var var_one = elt as VarDeclarationElement;
                                    if (vars != null)
                                    {
                                        foreach (var v in vars)
                                        {
                                            AddError(v.StartInfo, "Глобальная переменная " + v.ToString());
                                            КоличествоГлобальныхПеременных++;
                                        }
                                    }
                                    else if (var_one != null)
                                    {
                                        AddError(var_one.StartInfo, "Глобальная переменная " + var_one.ToString());
                                        КоличествоГлобальныхПеременных++;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        protected override void DoFinishAnalyse(EventArgs e)
        {
            //AddMetric("Количество глобальных переменных", КоличествоГлобальныхПеременных, CurFile);
            //КоличествоГлобальныхПеременных = 0;
            //CurFile = "";
        }
    }
}
