﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSyntaxAnalyser.SyntaxTree
{
    using daBase.Objects;

    /// <summary>
    /// Блок объявления констант
    /// </summary>
    public class ConstDefinitionsElement : SyntaxTreeElement, ISectionElementElement
    {
        private List<ConstDefinitionElement> consts;

        /// <summary>
        /// Объявления констант
        /// </summary>
        public List<ConstDefinitionElement> Consts { get { return consts; } set { consts = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="StartInfo">информация о позиции элемента в коде</param>
        /// <param name="Consts">объявления констант</param>
        /// <param name="Comments">комментарии, относящиеся к элементу</param>
        public ConstDefinitionsElement(TokenInfo StartInfo, List<ConstDefinitionElement> Consts, List<TokenInfo> Comments)
            : base(StartInfo, Comments)
        {
            consts = Consts;
        }

        /// <summary>
        /// Представление элемента в строковом формате
        /// </summary>
        /// <returns>строковое представление элемента</returns>
        public override string ToString()
        {
            return (consts != null) ? string.Format("const\r\n{0}", Tabulate(consts.Join("\r\n"))) : string.Empty;
        }

        /// <summary>
        /// Создание типизированного энумератора
        /// </summary>
        /// <returns>энумератор</returns>
        protected override IEnumerator<ISyntaxTreeElement> CreateEnumerator()
        {
            foreach (ConstDefinitionElement _const in consts)
                yield return _const;
        }
    }
}
