﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSyntaxAnalyser.SyntaxTree
{
    using daBase.Objects;

    /// <summary>
    /// Заголовок функции
    /// </summary>
    public class InnerFunctionHeadElement : InnerProcedureHeadElement
    {
        protected ITypeElement resulttype;

        /// <summary>
        /// Тип возвращаемого значения
        /// </summary>
        public ITypeElement ResultType { get { return resulttype; } set { resulttype = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="Type">тип процедуры (процедура, конструктор, деструктор)</param>
        /// <param name="Name">имя процедуры</param>
        /// <param name="Vars">список параметров процедуры</param>
        /// <param name="Directives">список директив процедуры</param>
        /// <param name="ResultType">тип возвращаемого значения</param>
        /// <param name="Comments">комментарии, относящиеся к элементу</param>
        public InnerFunctionHeadElement(TokenInfo Type, TokenInfo Name, List<FunctionVarDeclarationElement> Vars, List<DirectiveElement> Directives, ITypeElement ResultType, List<TokenInfo> Comments)
            : base(Type, Name, Vars, Directives, Comments)
        {
            resulttype = ResultType;
        }

        /// <summary>
        /// Представление элемента в строковом формате
        /// </summary>
        /// <returns>строковое представление элемента</returns>
        public override string ToString()
        {
            string result = string.Format("{0} {1}", type, id);
            if ((vars != null) && (vars.Count > 0))
                result += string.Format("({0})", vars.Join("; "));
            if (resulttype != null)
                result += string.Format(" : {0}", resulttype);
            if ((directives != null) && (directives.Count > 0))
                result += string.Format("; {0}", directives.Join("; "));
            return string.Format("{0};", result);
        }

        /// <summary>
        /// Создание типизированного энумератора
        /// </summary>
        /// <returns>энумератор</returns>
        protected override IEnumerator<ISyntaxTreeElement> CreateEnumerator()
        {
            if (vars != null)
                foreach (FunctionVarDeclarationElement var in vars)
                    yield return var;
            if (resulttype != null)
                yield return resulttype;
            if (directives != null)
                foreach (DirectiveElement directive in directives)
                    yield return directive;
        }
    }
}
