﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSyntaxAnalyser.SyntaxTree
{
    using daBase.Objects;

    /// <summary>
    /// Тип - ссылка на метод
    /// </summary>
    public class ReferenceTypeElement : SyntaxTreeElement, ITypeElement
    {
        private ProcedureTypeElement method;

        /// <summary>
        /// Метод
        /// </summary>
        public ProcedureTypeElement Method { get { return method; } set { method = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="StartInfo">информация о позиции элемента в коде</param>
        /// <param name="Type">метод</param>
        /// <param name="Comments">комментарии, относящиеся к элементу</param>
        public ReferenceTypeElement(TokenInfo StartInfo, ProcedureTypeElement Method, List<TokenInfo> Comments)
            : base(StartInfo, Comments)
        {
            method = Method;
        }

        /// <summary>
        /// Представление элемента в строковом формате
        /// </summary>
        /// <returns>строковое представление элемента</returns>
        public override string ToString()
        {
            return string.Format("reference to {0}", method);
        }

        /// <summary>
        /// Создание типизированного энумератора
        /// </summary>
        /// <returns>энумератор</returns>
        protected override IEnumerator<ISyntaxTreeElement> CreateEnumerator()
        {
            yield return method;
        }
    }
}
