﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSyntaxAnalyser.SyntaxTree
{
    using daBase.Objects;

    /// <summary>
    /// Блок объявлений типов
    /// </summary>
    public class TypeDeclarationsElement : SyntaxTreeElement, ISectionElementElement
    {
        private List<TypeDeclarationElement> declarations;

        /// <summary>
        /// Объявления типов
        /// </summary>
        public List<TypeDeclarationElement> Declarations { get { return declarations; } set { declarations = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="StartInfo">информация о позиции элемента в коде</param>
        /// <param name="Declarations">объявления типов</param>
        /// <param name="Comments">комментарии, относящиеся к элементу</param>
        public TypeDeclarationsElement(TokenInfo StartInfo, List<TypeDeclarationElement> Declarations, List<TokenInfo> Comments)
            : base(StartInfo, Comments)
        {
            declarations = Declarations;
        }

        /// <summary>
        /// Представление элемента в строковом формате
        /// </summary>
        /// <returns>строковое представление элемента</returns>
        public override string ToString()
        {
            return (declarations != null) ? string.Format("type\r\n{0}", Tabulate(declarations.Join("\r\n"))) : string.Empty;
        }

        /// <summary>
        /// Создание типизированного энумератора
        /// </summary>
        /// <returns>энумератор</returns>
        protected override IEnumerator<ISyntaxTreeElement> CreateEnumerator()
        {
            if (declarations != null)
                foreach (TypeDeclarationElement declaration in declarations)
                    yield return declaration;
        }
    }
}
