﻿using System;
using System.Collections.Generic;
using System.Text;

namespace daSyntaxAnalyser.SyntaxTree
{
    using daBase.Objects;

    /// <summary>
    /// Объявление переменной
    /// </summary>
    public class VarDeclarationElement : FunctionVarDeclarationElement
    {
        private IdElement absolute;

        /// <summary>
        /// ссылка absolute
        /// </summary>
        public IdElement Absolute { get { return absolute; } set { absolute = value; } }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="StartInfo">информация о позиции элемента в коде</param>
        /// <param name="Ids">иентификаторы</param>
        /// <param name="Type">тип</param>
        /// <param name="IsPlatform">наличие директивы platform</param>
        /// <param name="Value">значение</param>
        /// <param name="Absolute">ссылка absolute</param>
        /// <param name="Comments">комментарии, относящиеся к элементу</param>
        public VarDeclarationElement(TokenInfo StartInfo, List<IdElement> Ids, ITypeElement Type, bool IsPlatform, ISyntaxTreeElement Value, IdElement Absolute, List<TokenInfo> Comments)
            : base(StartInfo, Ids, Type, IsPlatform, Value, Comments)
        {
            absolute = Absolute;
        }

        /// <summary>
        /// Представление элемента в строковом формате
        /// </summary>
        /// <returns>строковое представление элемента</returns>
        public override string ToString()
        {
            return string.Format("{0}{1}{2}{3};", ids.Join(", "),
                (type != null) ? string.Format(":{0}", type) : string.Empty,
                (value != null) ? string.Format(" = {0}", value) : string.Empty,
                (absolute != null) ? string.Format(" absolute {0}", absolute) : string.Empty);
        }

        /// <summary>
        /// Создание типизированного энумератора
        /// </summary>
        /// <returns>энумератор</returns>
        protected override IEnumerator<ISyntaxTreeElement> CreateEnumerator()
        {
            IEnumerator<ISyntaxTreeElement> baseEnumerator = base.CreateEnumerator();
            while (baseEnumerator.MoveNext())
                yield return baseEnumerator.Current;
            if (absolute != null)
                yield return absolute;
        }
    }
}
